'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.extractImages = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _decoder = require('jpg-stream/decoder');

var _decoder2 = _interopRequireDefault(_decoder);

var _encoder = require('png-stream/encoder');

var _encoder2 = _interopRequireDefault(_encoder);

var _zlib = require('zlib');

var _zlib2 = _interopRequireDefault(_zlib);

var _stream = require('stream');

var _stream2 = _interopRequireDefault(_stream);

var _concatFrames = require('concat-frames');

var _concatFrames2 = _interopRequireDefault(_concatFrames);

var _streamToArray = require('stream-to-array');

var _streamToArray2 = _interopRequireDefault(_streamToArray);

var _swfTags = require('./swf-tags');

var _tagReaders = require('./tag-readers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var imageTagCodes = [_swfTags.SwfTags.DefineBits, _swfTags.SwfTags.DefineBitsJPEG2, _swfTags.SwfTags.DefineBitsJPEG3, _swfTags.SwfTags.DefineBitsLossless, _swfTags.SwfTags.DefineBitsLossless2, _swfTags.SwfTags.DefineBitsJPEG4];

var coverageFlag = false;
var coverage = coverageFlag ?
// eslint-disable-next-line no-console
function (msg) {
  return console.log('coverage: ' + msg);
} : function () {
  return undefined;
};

// each extractor either returns either a value or a Promise
var extractors = {};

// use this only for extractors that don't return a Promise
var define = function define(code, extractor) {
  extractors[code] = function (tagData, context) {
    return _extends({
      code: code
    }, extractor(tagData, context));
  };
};

var pngMagic = Buffer.from('0x89 0x50 0x4E 0x47 0x0D 0x0A 0x1A 0x0A'.split(' ').map(Number));
var gifMagic = Buffer.from('0x47 0x49 0x46 0x38 0x39 0x61'.split(' ').map(Number));
var recognizeHeader = function recognizeHeader(buffer) {
  if (pngMagic.equals(buffer.slice(0, pngMagic.length))) return 'png';
  if (gifMagic.equals(buffer.slice(0, gifMagic.length))) return 'gif';
  return 'jpeg';
};

define(_swfTags.SwfTags.DefineBits, function (tagData, context) {
  var characterId = tagData.characterId,
      jpegData = tagData.jpegData;

  var attachJpegTables = context.thunkAttachJpegTables();
  return {
    characterId: characterId,
    imgType: 'jpeg',
    imgData: attachJpegTables(jpegData)
  };
});

define(_swfTags.SwfTags.DefineBitsJPEG2, function (tagData) {
  var characterId = tagData.characterId,
      imageData = tagData.imageData;

  var imgType = recognizeHeader(imageData);
  return {
    characterId: characterId,
    imgType: imgType,
    imgData: imageData
  };
});

var gDefineBitsJPEG3or4Handler = function gDefineBitsJPEG3or4Handler(code) {
  return function (tagData) {
    var characterId = tagData.characterId,
        imageData = tagData.imageData;

    var imgType = recognizeHeader(imageData);
    if (imgType !== 'jpeg') {
      return {
        code: code,
        characterId: characterId,
        imgType: imgType,
        imgData: imageData
      };
    }

    var bitmapAlphaData = tagData.bitmapAlphaData;

    return new Promise(function (resolve, reject) {
      var enc = new _encoder2.default(undefined, undefined, { colorSpace: 'rgba' });
      _zlib2.default.unzip(bitmapAlphaData, function (err, alphaBufPre) {
        // INVARIANT: alphaBuf is either null or a non-empty buffer
        var alphaBuf = null;
        if (err) {
          /*
             Due to a bug present in node zlib (https://github.com/nodejs/node/issues/17041)
             unzipping an empty buffer can raise "unexpected end of file" error.
             We fix this here so that our impl does not depend on the version of node
             being used.
              Theoretically every zlib.unzip call needs to be guarded, but for this package,
             other two zlib.unzip call happens at sites that an empty uncompressed Buffer
             does not make sense. So I think the current fix is good enough.
           */
          if (bitmapAlphaData.length > 0) {
            return reject(new Error(err));
          }
          // leaving alphaBuf as null
        } else {
          // ensure alphaBuf is only assigned an non-empty Buffer
          if (alphaBufPre.length > 0) alphaBuf = alphaBufPre;
        }
        var bufferStream = new _stream2.default.PassThrough();
        bufferStream.end(imageData);
        bufferStream.pipe(new _decoder2.default()).pipe((0, _concatFrames2.default)(function (_ref) {
          var _ref2 = _slicedToArray(_ref, 1),
              frame = _ref2[0];

          var input = frame.pixels;
          var pCount = frame.width * frame.height;
          var output = new Buffer(pCount * 4);
          if (alphaBuf !== null && alphaBuf.length !== pCount) {
            console.error('expect alphaBuf to have size ' + pCount + ' while getting ' + alphaBuf.length);
          }
          var getAlphaBuf = alphaBuf === null ? function (_ignored) {
            return 0xff;
          } : function (i) {
            return alphaBuf[i];
          };

          for (var i = 0; i < pCount; ++i) {
            output[4 * i] = input[3 * i];
            output[4 * i + 1] = input[3 * i + 1];
            output[4 * i + 2] = input[3 * i + 2];
            output[4 * i + 3] = getAlphaBuf(i);
          }
          enc.format.width = frame.width;
          enc.format.height = frame.height;
          enc.end(output);
        }));
      });
      (0, _streamToArray2.default)(enc).then(function (parts) {
        var buffers = parts.map(function (part) {
          return Buffer.isBuffer(part) ? part : Buffer.from(part);
        });
        resolve({
          code: code,
          characterId: characterId,
          imgType: 'png',
          imgData: Buffer.concat(buffers)
        });
      });
    });
  };
};

extractors[_swfTags.SwfTags.DefineBitsJPEG3] = gDefineBitsJPEG3or4Handler(_swfTags.SwfTags.DefineBitsJPEG3);
extractors[_swfTags.SwfTags.DefineBitsJPEG4] = gDefineBitsJPEG3or4Handler(_swfTags.SwfTags.DefineBitsJPEG4);

extractors[_swfTags.SwfTags.DefineBitsLossless] = function (tagData) {
  return new Promise(function (resolve, reject) {
    var characterId = tagData.characterId,
        bitmapFormat = tagData.bitmapFormat,
        bitmapWidth = tagData.bitmapWidth,
        bitmapHeight = tagData.bitmapHeight,
        bitmapColorTableSize = tagData.bitmapColorTableSize,
        zlibBitmapData = tagData.zlibBitmapData;


    var enc = new _encoder2.default(bitmapWidth, bitmapHeight, { colorSpace: 'rgb' });
    _zlib2.default.unzip(zlibBitmapData, function (err, dataBuf) {
      if (err) return reject(new Error(err));
      var output = new Buffer(bitmapWidth * bitmapHeight * 3);
      /* eslint-disable no-bitwise */
      if (
      // 15-bit RGB image
      bitmapFormat === 4 ||
      // 24-bit RGB image
      bitmapFormat === 5) {
        if (bitmapFormat === 4) coverage('DefineBitsLossless 15-bit');
        var index = 0;
        var ptr = 0;
        for (var y = 0; y < bitmapHeight; ++y) {
          for (var x = 0; x < bitmapWidth; ++x) {
            if (bitmapFormat === 4) {
              // 15-bit RGB image
              var val = dataBuf[ptr] << 8 + dataBuf[ptr + 1];
              // pix15red
              output[index] = (val & 0x7c00) >> 10;
              // pix15green
              output[index + 1] = (val & 0x3e0) >> 5;
              // pix15blue
              output[index + 2] = val & 0x1f;
              ptr += 2;
              index += 3;
            } else {
              // 24-bit RGB image
              ++ptr; // skip reversed byte
              output[index] = dataBuf[ptr];
              output[index + 1] = dataBuf[ptr + 1];
              output[index + 2] = dataBuf[ptr + 2];
              ptr += 3;
              index += 3;
            }
          }
          if (bitmapWidth % 2 !== 0) {
            ptr += 2; // skip padding
          }
        }
      } else if (bitmapFormat === 3) {
        // 8-bit colormapped image
        var colorMap = [];
        var _index = 0;
        var _ptr = 0;
        for (var i = 0; i < bitmapColorTableSize + 1; ++i) {
          colorMap.push([dataBuf[_ptr], dataBuf[_ptr + 1], dataBuf[_ptr + 2]]);
          _ptr += 3;
        }
        for (var _y = 0; _y < bitmapHeight; ++_y) {
          for (var _x = 0; _x < bitmapWidth; ++_x) {
            var idx = dataBuf[_ptr];
            ++_ptr;
            var color = idx < colorMap.length ? colorMap[idx] : [0, 0, 0];
            output[_index] = color[0];
            output[_index + 1] = color[1];
            output[_index + 2] = color[2];
            _index += 3;
          }
          // skip padding
          _ptr += (4 - bitmapWidth % 4) % 4;
        }
      } else {
        return reject(new Error('unhandled bitmapFormat: ' + bitmapFormat));
      }
      /* eslint-enable no-bitwise */
      enc.end(output);
    });

    (0, _streamToArray2.default)(enc).then(function (parts) {
      var buffers = parts.map(function (part) {
        return Buffer.isBuffer(part) ? part : Buffer.from(part);
      });
      resolve({
        code: _swfTags.SwfTags.DefineBitsLossless,
        characterId: characterId,
        imgType: 'png',
        imgData: Buffer.concat(buffers)
      });
    });
  });
};

extractors[_swfTags.SwfTags.DefineBitsLossless2] = function (tagData) {
  var characterId = tagData.characterId,
      bitmapFormat = tagData.bitmapFormat,
      bitmapWidth = tagData.bitmapWidth,
      bitmapHeight = tagData.bitmapHeight,
      bitmapColorTableSize = tagData.bitmapColorTableSize,
      zlibBitmapData = tagData.zlibBitmapData;


  return new Promise(function (resolve, reject) {
    var enc = new _encoder2.default(bitmapWidth, bitmapHeight, { colorSpace: 'rgba' });

    _zlib2.default.unzip(zlibBitmapData, function (err, dataBuf) {
      if (err) {
        return reject(new Error(err));
      }
      var output = new Buffer(bitmapWidth * bitmapHeight * 4);
      var index = 0;
      var ptr = 0;
      if (bitmapFormat === 5) {
        // 32-bit ARGB image
        for (var y = 0; y < bitmapHeight; ++y) {
          for (var x = 0; x < bitmapWidth; ++x) {
            var alpha = dataBuf[ptr];
            output[index] = dataBuf[ptr + 1];
            output[index + 1] = dataBuf[ptr + 2];
            output[index + 2] = dataBuf[ptr + 3];
            output[index + 3] = alpha;
            index += 4;
            ptr += 4;
          }
        }
      } else if (bitmapFormat === 3) {
        // 8-bit colormapped image
        var colorMap = [];
        for (var i = 0; i < bitmapColorTableSize + 1; ++i) {
          colorMap.push([dataBuf[ptr], dataBuf[ptr + 1], dataBuf[ptr + 2], dataBuf[ptr + 3]]);
          ptr += 4;
        }
        for (var _y2 = 0; _y2 < bitmapHeight; ++_y2) {
          for (var _x2 = 0; _x2 < bitmapWidth; ++_x2) {
            var idx = dataBuf[ptr];
            var color = idx < colorMap.length ? colorMap[idx] : [0, 0, 0, 0];
            output[index] = color[0];
            output[index + 1] = color[1];
            output[index + 2] = color[2];
            output[index + 3] = color[3];
            ptr += 1;
            index += 4;
          }
          // skip padding
          ptr += (4 - bitmapWidth % 4) % 4;
        }
      } else {
        return reject(new Error('unhandled bitmapFormat: ' + bitmapFormat));
      }
      enc.end(output);
    });

    (0, _streamToArray2.default)(enc).then(function (parts) {
      var buffers = parts.map(function (part) {
        return Buffer.isBuffer(part) ? part : Buffer.from(part);
      });
      resolve({
        code: _swfTags.SwfTags.DefineBitsLossless2,
        characterId: characterId,
        imgType: 'png',
        imgData: Buffer.concat(buffers)
      });
    });
  }).catch(function (e) {
    console.error(e);
  });
};

var mkContext = function mkContext(rawTags) {
  return {
    // call without argument for the memoization to work.
    // returns a thunk that evaluates a function which can be used to
    // attach the JPEGTables tag in front of a DefineBits tag
    thunkAttachJpegTables: _lodash2.default.memoize(function () {
      var jpegTablesCandidates = rawTags.filter(function (rt) {
        return rt.code === _swfTags.SwfTags.JPEGTables;
      });

      if (jpegTablesCandidates.length > 1) {
        console.warn('There may only be one JPEGTables tag in a SWF file');
        return _lodash2.default.identity;
      }

      if (jpegTablesCandidates.length === 0) {
        console.warn('Expecting a JPEGTables tag but found none');
        return _lodash2.default.identity;
      }

      var _jpegTablesCandidates = _slicedToArray(jpegTablesCandidates, 1),
          jpegTablesRawTag = _jpegTablesCandidates[0];

      var jpegTablesTag = _tagReaders.tagReaders[_swfTags.SwfTags.JPEGTables](jpegTablesRawTag.rawData);

      // nothing to attach if JPEGTables turns out to be empty
      if (jpegTablesTag.jpegData.length === 0) {
        return _lodash2.default.identity;
      } else {
        coverage('JPEGTables non-empty');
        return function (imgBuffer) {
          return Buffer.concat([jpegTablesTag.jpegData, imgBuffer]);
        };
      }
    })
  };
};

/*
   extracts image data Buffer from raw tags.

   tags are assume to be an Object of at least following structure:
   - code: number
   - rawData: Buffer that contains raw data of that tag, starts right after
     the length field of a tag.

   returns an Array of Promises for every image tag

   every resolved result is guaranteed to have the following structure:

   - code: number
   - characterId: number
   - imgType: jpg / png / gif
   - imgData: Buffer

 */
var extractImages = function extractImages(rawTags) {
  var context = mkContext(rawTags);

  return rawTags.filter(function (t) {
    return imageTagCodes.includes(t.code);
  }).map(function (rawTag) {
    return new Promise(function (resolve) {
      // parse tag into structures as described by spec
      var tag = _tagReaders.tagReaders[rawTag.code](rawTag.rawData);
      Promise.resolve(extractors[tag.code](tag, context)).then(resolve);
    });
  });
};

exports.extractImages = extractImages;